/*
 Author : 148574B Kaito Tokumori
 Haar Transform Program
*/

package main

import(
	"fmt"
	"math/big"
	"bytes"
	"math"
)

type RatWithRoot struct {
	rat *big.Rat
	root *big.Rat
}

func (v *RatWithRoot) mulRoot(i *big.Rat) {
	if v.root.Cmp(i) == 0 {
		v.rat.Mul(i,v.root)
		v.root = big.NewRat(1,1)
	} else {
		v.root.Mul(i, v.root)
	}
}

func (v *RatWithRoot) mulRat(r *big.Rat) {
	v.rat.Mul(r, v.rat)
}

func (v *RatWithRoot) mulInt(i int64) {
	v.rat.Mul(big.NewRat(i,1), v.rat)
}

func (v *RatWithRoot) String() string {
	one := big.NewInt(1)
	var buf bytes.Buffer
	buf.WriteString(fmt.Sprint(v.rat.Num(),"R",v.root.Num()))
	if one.Cmp(v.rat.Denom()) != 0 || one.Cmp(v.root.Denom()) != 0 {
		buf.WriteString(fmt.Sprint("/",v.rat.Denom(),"R",v.root.Denom()))
	}
	return buf.String()
	
}


func NewRWR(a,b int64) *RatWithRoot{
	return &RatWithRoot{big.NewRat(a,b),big.NewRat(1,1)}
}

func main(){
	N := 4
	Z := []float64 {0,0.75}
	outSignal := make([][]*RatWithRoot, N, N)
	for s := 0; s < N; s++ {
		outSignal[s] = make([]*RatWithRoot, N, N)
	}
	for i := 0; i<N; i++ {
		for j := 0; j<N; j++ {
			outSignal[i][j] = NewRWR(1,1)
		}
	}
	var k = [4]map[string] float64 {
		{"p":0,"q":0,},
		{"p":0,"q":1,},
		{"p":1,"q":1,},
		{"p":1,"q":2,},
	}
	for i := 0; i<N; i++ {
		for j, zNum := 0, Z[0]; j<N; j, zNum = j+1, zNum+Z[1] {
			if i==0 {
				continue
			}
			z := zNum/float64(N)
			if (k[i]["q"]-1) / math.Pow(2,k[i]["p"]) <= z && z < (k[i]["q"]-1.0/2.0) / math.Pow(2,k[i]["p"]) {
				outSignal[i][j].mulInt((int64)(math.Pow(2, (float64)((int)(k[i]["p"]/2)))))
				if ((int)(k[i]["p"]) % 2 != 0) {
					outSignal[i][j].mulRoot(big.NewRat(2,1))
				}
			} else if (k[i]["q"]-1.0/2.0) / math.Pow(2,k[i]["p"]) <= z && z < k[i]["q"] / math.Pow(2,k[i]["p"]) {
				outSignal[i][j].mulInt( (int64)(math.Pow(2, (float64)((int)(k[i]["p"]/2)))))
				outSignal[i][j].mulInt(-1)
				if ((int)(k[i]["p"]) % 2 != 0) {
					outSignal[i][j].mulRoot(big.NewRat(2,1))
				} 
			} else {
				outSignal[i][j].mulInt(0)
			}
		}
	}
	showHaarMatrix(outSignal,N)
}

func showHaarMatrix(mat [][]*RatWithRoot, N int) {
	fmt.Printf("coefficient : 1/R%d\n",N)
	fmt.Printf("matrix : [\n")
	for _, v := range mat {
		fmt.Println(v)
	}
	fmt.Printf("]\n")
}
